/*
 *  freesmartphone-phonebook.c
 *
 *  Copyright(C) 2008 Ixonos Plc
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Boston, MA 02111.
 */

/*
 * Written by
 *   Heikki Paajanen
 *   Vesa Pikki
 */

#include "freesmartphone-internal.h"
#include "freesmartphone-phonebook.h"
#include "freesmartphone-phonebook-binding.h"
#include "fso-marshallers.h"



/**
* @brief Initializes Phonebook interface, it is called automatically
* when necessary
*
* @param fs pointer to freesmartphone struct
*/
void fso_gsm_phonebook_init(FreeSmartphone *fs)
{
        g_debug("%s", __func__);
}


/**
* @brief Reply function to fso_gsm_phonebook_get_storage_spaces function call
*
* @param proxy dbus proxy used to call fso_gsm_phonebook_get_storage_spaces
* function
* @param spaces storage spaces
* @param error pointer to error struct
* @param userdata userdata passed to fso_gsm_phonebook_get_storage_spaces
*/
static
void fso_gsm_phonebook_get_storage_spaces_reply(DBusGProxy *proxy,
                                                gchar **spaces,
                                                GError *error,
                                                gpointer userdata)
{
        g_debug("%s", __func__);
        ReplyData *reply = (ReplyData*)userdata;
        FSGSMPhonebookGetStorageSpacesReplyFunc reply_cb = (FSGSMPhonebookGetStorageSpacesReplyFunc)reply->cb;
        gpointer data = reply->data;
        fso_reply_data_free(reply);
        reply_cb(spaces,error,data);
}


/**
* @brief Get storage spaces
*
* @param fs pointer to freesmartphone struct
* @param cb callback function to call when the command
* has finished
* @param userdata user data
* @return true if function was succesfully executed
*/
gboolean fso_gsm_phonebook_get_storage_spaces(FreeSmartphone *fs,
                                              FSGSMPhonebookGetStorageSpacesReplyFunc cb,
                                              gpointer userdata)
{
        g_return_val_if_fail(fs, FALSE);
        g_debug("%s", __func__);
        ReplyData *replydata = fso_prepare_method(fs,
                                                  G_CALLBACK(cb),
                                                  userdata,
                                                  FSO_PHONEBOOK);
        if ( !replydata )
                return FALSE;

        DBusGProxy *proxy = fso_get_proxy(fs,FSO_PHONEBOOK);

        org_freesmartphone_GSM_Phonebook_get_storage_spaces_async(proxy,
                                                                  &fso_gsm_phonebook_get_storage_spaces_reply,
                                                                  (gpointer)replydata);
        return TRUE;
}


/**
* @brief Reply function to fso_gsm_pdp_list_gprs_classes function call
*
* @param proxy dbus proxy used to call fso_gsm_phonebook_get_default_storage_space
* function
* @param space default storage space
* @param error pointer to error struct
* @param userdata userdata passed to fso_gsm_phonebook_get_default_storage_space
*/
static
void fso_gsm_phonebook_get_default_storage_space_reply(DBusGProxy *proxy,
                                                       gchar *space,
                                                       GError *error,
                                                       gpointer userdata)
{
        g_debug("%s", __func__);
        ReplyData *reply = (ReplyData*)userdata;
        FSGSMPhonebookGetDefaultStorageSpaceReplyFunc reply_cb = (FSGSMPhonebookGetDefaultStorageSpaceReplyFunc)reply->cb;
        gpointer data = reply->data;
        fso_reply_data_free(reply);
        reply_cb(space,error,data);
}


/**
* @brief Get default storage space
*
* @param fs pointer to freesmartphone struct
* @param cb callback function to call when the command
* has finished
* @param userdata user data
* @return true if function was succesfully executed
*/
gboolean fso_gsm_phonebook_get_default_storage_space(FreeSmartphone *fs,
                                                     FSGSMPhonebookGetDefaultStorageSpaceReplyFunc cb,
                                                     gpointer userdata)
{
        g_return_val_if_fail(fs, FALSE);
        g_debug("%s", __func__);
        ReplyData *replydata = fso_prepare_method(fs,
                                                  G_CALLBACK(cb),
                                                  userdata,
                                                  FSO_PHONEBOOK);
        if ( !replydata )
                return FALSE;

        DBusGProxy *proxy = fso_get_proxy(fs,FSO_PHONEBOOK);

        org_freesmartphone_GSM_Phonebook_get_default_storage_space_async(proxy,
                                                                         &fso_gsm_phonebook_get_default_storage_space_reply,
                                                                         (gpointer)replydata);
        return TRUE;
}


/**
* @brief General reply function to methods with no return values
*
* @param proxy dbus proxy used
* @param error pointer to error struct
* @param userdata user data
*/
static
void fso_gsm_phonebook_reply(DBusGProxy *proxy,
                             GError *error,
                             gpointer userdata)
{
        g_debug("%s", __func__);
        ReplyData *reply = (ReplyData*)userdata;
        FSGSMPhonebookReplyFunc reply_cb = (FSGSMPhonebookReplyFunc)reply->cb;
        gpointer data = reply->data;
        fso_reply_data_free(reply);
        reply_cb(error,data);
}


/**
* @brief set default storage space
*
* @param fs pointer to freesmartphone struct
* @param cb callback function to call when the command
* has finished
* @param userdata user data
* @return true if function was succesfully executed
*/
gboolean fso_gsm_phonebook_set_default_storage_space(FreeSmartphone *fs,
                                                     gchar *space,
                                                     FSGSMPhonebookReplyFunc cb,
                                                     gpointer userdata)
{
        g_return_val_if_fail(fs, FALSE);
        g_debug("%s", __func__);
        ReplyData *replydata = fso_prepare_method(fs,
                                                  G_CALLBACK(cb),
                                                  userdata,
                                                  FSO_PHONEBOOK);
        if ( !replydata )
                return FALSE;

        DBusGProxy *proxy = fso_get_proxy(fs,FSO_PHONEBOOK);

        org_freesmartphone_GSM_Phonebook_set_default_storage_space_async(proxy,
                                                                         space,
                                                                         &fso_gsm_phonebook_reply,
                                                                         (gpointer)replydata);
        return TRUE;
}


/**
* @brief Reply function to fso_gsm_phonebook_get_index_boundaries function call
*
* @param proxy dbus proxy used to call fso_gsm_phonebook_get_index_boundaries
* function
* @param error pointer to error struct
* @param userdata userdata passed to fso_gsm_phonebook_get_index_boundaries
*/
static
void fso_gsm_phonebook_get_index_boundaries_reply(DBusGProxy *proxy,
                                                  gint lowest_index,
                                                  gint highest_index,
                                                  GError *error,
                                                  gpointer userdata)
{
        g_debug("%s", __func__);
        ReplyData *reply = (ReplyData*)userdata;
        FSGSMPhonebookGetIndexBoundariesReplyFunc reply_cb = (FSGSMPhonebookGetIndexBoundariesReplyFunc)reply->cb;
        gpointer data = reply->data;
        fso_reply_data_free(reply);
        reply_cb(lowest_index,highest_index,error,data);
}


/**
* @brief Get index boundaries
*
* @param fs pointer to freesmartphone struct
* @param cb callback function to call when the command
* has finished
* @param userdata user data
* @return true if function was succesfully executed
*/
gboolean fso_gsm_phonebook_get_index_boundaries(FreeSmartphone *fs,
                                                FSGSMPhonebookGetIndexBoundariesReplyFunc cb,
                                                gpointer userdata)
{
        g_return_val_if_fail(fs, FALSE);
        g_debug("%s", __func__);
        ReplyData *replydata = fso_prepare_method(fs,
                                                  G_CALLBACK(cb),
                                                  userdata,
                                                  FSO_PHONEBOOK);
        if ( !replydata )
                return FALSE;

        DBusGProxy *proxy = fso_get_proxy(fs,FSO_PHONEBOOK);

        org_freesmartphone_GSM_Phonebook_get_index_boundaries_async(proxy,
                                                                    &fso_gsm_phonebook_get_index_boundaries_reply,
                                                                    (gpointer)replydata);
        return TRUE;
}


/**
* @brief Delete entry
*
* @param fs pointer to freesmartphone struct
* @param cb callback function to call when the command
* has finished
* @param userdata user data
* @return true if function was succesfully executed
*/
gboolean fso_gsm_phonebook_delete_entry(FreeSmartphone *fs,
                                        gint index,
                                        FSGSMPhonebookReplyFunc cb,
                                        gpointer userdata)
{
        g_return_val_if_fail(fs, FALSE);
        g_debug("%s", __func__);
        ReplyData *replydata = fso_prepare_method(fs,
                                                  G_CALLBACK(cb),
                                                  userdata,
                                                  FSO_PHONEBOOK);
        if ( !replydata )
                return FALSE;

        DBusGProxy *proxy = fso_get_proxy(fs,FSO_PHONEBOOK);

        org_freesmartphone_GSM_Phonebook_delete_entry_async(proxy,
                                                            index,
                                                            &fso_gsm_phonebook_reply,
                                                            (gpointer)replydata);
        return TRUE;
}




/**
* @brief Store entry
*
* @param fs pointer to freesmartphone struct
* @param cb callback function to call when the command
* has finished
* @param userdata user data
* @return true if function was succesfully executed
*/
gboolean fso_gsm_phonebook_store_entry(FreeSmartphone *fs,
                                       gint index,
                                       gint type,
                                       gchar *name,
                                       gchar *contents,
                                       FSGSMPhonebookReplyFunc cb,
                                       gpointer userdata)
{
        g_return_val_if_fail(fs, FALSE);
        g_debug("%s", __func__);
        ReplyData *replydata = fso_prepare_method(fs,
                                                  G_CALLBACK(cb),
                                                  userdata,
                                                  FSO_PHONEBOOK);
        if ( !replydata )
                return FALSE;

        DBusGProxy *proxy = fso_get_proxy(fs,FSO_PHONEBOOK);

        org_freesmartphone_GSM_Phonebook_store_entry_async(proxy,
                                                           index,
                                                           type,
                                                           name,
                                                           contents,
                                                           &fso_gsm_phonebook_reply,
                                                           (gpointer)replydata);
        return TRUE;
}


/**
* @brief Reply function to fso_gsm_phonebook_retrieve_entry function call
*
* @param proxy dbus proxy used to call fso_gsm_phonebook_retrieve_entry
* function
* @param spaces storage spaces
* @param error pointer to error struct
* @param userdata userdata passed to fso_gsm_phonebook_retrieve_entry
*/
static
void fso_gsm_phonebook_retrieve_entry_reply(DBusGProxy *proxy,
                                            gint type,
                                            gchar *name,
                                            gchar *contents,
                                            GError *error,
                                            gpointer userdata)
{
        g_debug("%s", __func__);
        ReplyData *reply = (ReplyData*)userdata;
        FSGSMPhonebookRetrieveEntryReplyFunc reply_cb = (FSGSMPhonebookRetrieveEntryReplyFunc)reply->cb;
        gpointer data = reply->data;
        fso_reply_data_free(reply);
        reply_cb(type,name,contents,error,data);
}


/**
* @brief Retrieve entry
*
* @param fs pointer to freesmartphone struct
* @param cb callback function to call when the command
* has finished
* @param userdata user data
* @return true if function was succesfully executed
*/
gboolean fso_gsm_phonebook_retrieve_entry(FreeSmartphone *fs,
                                          gint index,
                                          FSGSMPhonebookRetrieveEntryReplyFunc cb,
                                          gpointer userdata)
{
        g_return_val_if_fail(fs, FALSE);
        g_debug("%s", __func__);
        ReplyData *replydata = fso_prepare_method(fs,
                                                  G_CALLBACK(cb),
                                                  userdata,
                                                  FSO_PHONEBOOK);
        if ( !replydata )
                return FALSE;

        DBusGProxy *proxy = fso_get_proxy(fs,FSO_PHONEBOOK);

        org_freesmartphone_GSM_Phonebook_retrieve_entry_async(proxy,
                                                              index,
                                                              &fso_gsm_phonebook_retrieve_entry_reply,
                                                              (gpointer)replydata);
        return TRUE;
}
