/*
 *  phonebook.c
 *
 *  Copyright(C) 2007,2008 Ixonos Plc
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Boston, MA 02111.
 */

/*
 * Written by
 *   Jukka Honkela
 *   Yi Zheng
 *   Matti Katila
 *   Vesa Pikki
 *   Heikki Paajanen
 */

#include "phonebook.h"
#include "gsmd-error.h"
#include "utils.h"
#include <stdlib.h>
#include "modem.h"
static void gsmd_phonebook_storage_spaces_free(AtCommandContext *at);


static const AtCommand
phonebook_commands[]
= {
        //Get available storage space
        { PHONEBOOK_SPACES_QUERY,         "AT+CPBS=?\r\n",            100,
                FALSE,  0,
                gsmd_phonebook_handler_get_storage_spaces, NULL, SIM_READY,
                gsmd_phonebook_storage_spaces_free},

        //Get current storage space
        { PHONEBOOK_CUR_SPACE_QUERY,      "AT+CPBS?\r\n",             100,
          FALSE,  0,
          gsmd_phonebook_handler_get_default_storage_space, NULL, SIM_READY,
          NULL},

        //Set storage space
        { PHONEBOOK_SPACE_SET,            "AT+CPBS=%s\r\n",           100,
          FALSE,  0,
          gsmd_phonebook_handler_set_default_storage_space, NULL, SIM_READY,
          NULL},

        //Get storage item's index boundaries
        { PHONEBOOK_GET_INDEX_BOUNDARIES, "AT+CPBR=?\r\n",            100,
          FALSE,  0,
          gsmd_phonebook_handler_get_index_boundaries, NULL, SIM_READY, NULL},

        //Get entry from storage
        { PHONEBOOK_ENTRY_GET,            "AT+CPBR=%s\r\n",           100,
          FALSE,  0,
          gsmd_phonebook_handler_retrieve_entry, NULL, SIM_READY, NULL},

        //Set storage entry
        { PHONEBOOK_ENTRY_SET,            "AT+CPBW=%s\r\n",           100,
          FALSE,  0,
          gsmd_phonebook_handler_store_entry, NULL, SIM_READY, NULL},

        //Delete storage entry
        { PHONEBOOK_ENTRY_DELETE,         "AT+CPBW=%s\r\n",           100,
          FALSE,  0,
          gsmd_phonebook_handler_delete_entry, NULL, SIM_READY, NULL},


        { 0,                    NULL,                100,    TRUE,  1,
          NULL, NULL, SIM_UNKNOWN, NULL },

}, *phonebook_command_p = phonebook_commands;

static const SymbolTable
phonebook_symbols[] = {
        { NULL, 0,                      },
}, *phonebook_symbol_p = phonebook_symbols;

void gsmd_phonebook_init_at_handler(ModemInterface* modem)
{
        g_debug("%s", __func__);

        while (phonebook_symbol_p->symbol_name) {
                g_scanner_add_symbol (modem->scanner, phonebook_symbol_p->symbol_name,
                                      GINT_TO_POINTER(phonebook_symbol_p->symbol_token));
                phonebook_symbol_p++;
        }
        while (phonebook_command_p->command) {
                gsmd_modem_register_command (modem,phonebook_command_p);
                phonebook_command_p++;
        }
}



void gsmd_phonebook_free_storage_spaces_array(GHashTable *table)
{
        int i=0;
        GValue *val = g_hash_table_lookup(table,GSMD_PHONEBOOK_KEY_STORAGE_SPACES);
        if (val) {
                GArray *array = g_value_get_pointer(val);

                for (i=0;i<array->len;i++) {
                        g_string_free(g_array_index(array,
                                                    GString*,
                                                    i),
                                      TRUE);
                }
                g_array_free(array,TRUE);


        }
}

static void gsmd_phonebook_storage_spaces_free(AtCommandContext *at)
{
        gsmd_phonebook_free_storage_spaces_array(at->handler_data);
}

AtCommandHandlerStatus
gsmd_phonebook_handler_get_storage_spaces(ModemInterface *modem,
                                          AtCommandContext *at,
                                          GString *response)
{
        /*
        AT+CPBS=?

        +CPBS: ("SM","FD","LD","MC","RC")

        OK
        */


        AtCommandHandlerStatus status = AT_HANDLER_DONT_UNDERSTAND;
        GError *error = NULL;
        GRegex *regex = g_regex_new ("(?<cpbs>\\+CPBS:\\s\\((\"\\w+\",?)+\\))|(?<error>ERROR)|(?<ok>OK)", 0, 0, NULL);
        GMatchInfo *match_info;

        if (g_regex_match (regex, response->str, 0, &match_info)) {
                if (gsmd_utils_match_exists(match_info,"cpbs")) {
                        g_free(error);
                        g_match_info_free (match_info);
                        g_regex_unref (regex);
                        error = NULL;
                        match_info = NULL;


                        regex = g_regex_new ("\"(?<id>\\w+)\"", 0, 0, NULL);

                        if (g_regex_match (regex, response->str, 0, &match_info)) {
                                GString *line = NULL;


                                GArray *array = g_array_new(FALSE,
                                                            TRUE,
                                                            sizeof(GString*));
                                gchar *match = NULL;

                                while (g_match_info_matches (match_info)) {
                                        match = g_match_info_fetch_named(match_info,"id");
                                        line = g_string_new(match);
                                        g_free(match);
                                        g_array_append_val(array,line);
                                        g_match_info_next (match_info, NULL);
                                }

                                gsmd_utils_table_insert_pointer(at->handler_data,
                                                                GSMD_PHONEBOOK_KEY_STORAGE_SPACES,
                                                                array);

                        }

                        status = AT_HANDLER_NEED_MORE;

                } else if (gsmd_utils_match_exists(match_info,"error")) {
                        gsmd_utils_send_error(at,
                                              GSMD_ERROR_UNKNOWN,
                                              "FAILED TO GET STORAGE SPACES");

                        status = AT_HANDLER_DONE_ERROR;
                } else if (gsmd_utils_match_exists(match_info,"ok")) {
                        GValue *val = g_hash_table_lookup(at->handler_data,
                                                          GSMD_PHONEBOOK_KEY_STORAGE_SPACES);

                        if (!val) {
                                g_warning("%s : Invalid handler data. storage spaces: %p",
                                          __func__,
                                          val);

                                gsmd_utils_send_error(at,
                                                      GSMD_ERROR_UNKNOWN,
                                                      "INVALID HANDLER DATA");
                                status = AT_HANDLER_DONE_ERROR;
                        } else {
                                gsmd_phonebook_free_storage_spaces_array(modem->cache);
                                g_hash_table_remove(modem->cache,GSMD_PHONEBOOK_KEY_STORAGE_SPACES);

                                modem->phonebook_ipc->get_storage_spaces_reply(modem->phonebook_ipc,
                                                                               at->ipc_data,
                                                                               g_value_get_pointer(val));
                                gsmd_utils_table_insert_pointer(modem->cache,
                                                                GSMD_PHONEBOOK_KEY_STORAGE_SPACES,
                                                                g_value_get_pointer(val));
                                g_hash_table_remove(at->handler_data,GSMD_PHONEBOOK_KEY_STORAGE_SPACES);
                                status = AT_HANDLER_DONE;
                        }
                }


        }

        g_free(error);
        g_match_info_free (match_info);
        g_regex_unref (regex);


        return status;
}


void gsmd_phonebook_command_get_storage_spaces(PhonebookInterface *phonebook,
                                               gpointer ipc_data)
{
        ModemInterface *modem = (ModemInterface*)phonebook->priv;
        GValue *val = g_hash_table_lookup(modem->cache,GSMD_PHONEBOOK_KEY_STORAGE_SPACES);

        if (!modem->no_cache && val) {
                modem->phonebook_ipc->get_storage_spaces_reply(modem->phonebook_ipc,
                                                               ipc_data,
                                                               g_value_get_pointer(val));
        } else {

                gsmd_modem_post_at_command_id( modem,
                                               PHONEBOOK_SPACES_QUERY,
                                               NULL,
                                               ipc_data,
                                               (ErrorFunction)modem->phonebook_ipc->get_storage_spaces_error,
                                               (gpointer)modem->phonebook_ipc,
                                               INTERFACE_PHONEBOOK,
                                               NULL);
        }
}

void gsmd_phonebook_command_get_storage_info(PhonebookInterface *phonebook,
                                             gpointer ipc_data)
{
        /*
                ModemInterface *modem = (ModemInterface*)phonebook->priv;
                const char *info[] = {"First","Second","Third"};
                modem->phonebook_ipc->get_storage_info_reply(modem->phonebook_ipc,ipc_data,info);
                */
}



AtCommandHandlerStatus gsmd_phonebook_handler_get_default_storage_space(ModemInterface *modem,
                                                                        AtCommandContext *at,
                                                                        GString *response)
{
        /*
        AT+CPBS?

        +CPBS: "SM",2,250

        OK

        \+CPBS:\s"(\w+)",\d+,\d+
        */
        AtCommandHandlerStatus status = AT_HANDLER_DONT_UNDERSTAND;
        GError **error = NULL;
        GRegex *regex = g_regex_new ("\\+CPBS:\\s\"(?<space>\\w+)\",\\d+,\\d+|(?<error>ERROR)|(?<ok>OK)", 0, 0, error);
        GMatchInfo *match_info;


        if (g_regex_match (regex, response->str, 0, &match_info)) {
                if (gsmd_utils_match_exists(match_info,"space")) {
                        gchar *match = g_match_info_fetch_named(match_info,"space");
                        gsmd_utils_table_insert_string(at->handler_data,GSMD_PHONEBOOK_KEY_DEFAULT_STORAGE_SPACE,match);
                        g_free(match);

                        status = AT_HANDLER_NEED_MORE;
                }

                if (gsmd_utils_match_exists(match_info,"error")) {
                        gsmd_utils_send_error(at,
                                              GSMD_ERROR_UNKNOWN,
                                              "FAILED TO GET DEFAULT PHONEBOOK STORAGE SPACE");
                        status = AT_HANDLER_DONE_ERROR;
                }

                if (gsmd_utils_match_exists(match_info,"ok")) {
                        GValue *val = g_hash_table_lookup(at->handler_data,
                                                          GSMD_PHONEBOOK_KEY_DEFAULT_STORAGE_SPACE);
                        if (!val) {
                                g_warning("%s : Invalid handler data. default storage space: %p",
                                          __func__,
                                          val);

                                gsmd_utils_send_error(at,
                                                      GSMD_ERROR_UNKNOWN,
                                                      "INVALID HANDLER DATA");
                        }
                        modem->phonebook_ipc->get_default_storage_space_reply(modem->phonebook_ipc,
                                                                              at->ipc_data,
                                                                              g_value_get_string(val));

                        gsmd_utils_table_insert_copy(modem->cache,
                                                     GSMD_PHONEBOOK_KEY_DEFAULT_STORAGE_SPACE,
                                                     val);

                        status = AT_HANDLER_DONE;
                }

        }

        g_free(error);
        g_match_info_free (match_info);
        g_regex_unref (regex);


        return status;
}


void gsmd_phonebook_command_get_default_storage_space(PhonebookInterface *phonebook,
                                                      gpointer ipc_data)
{
        ModemInterface *modem = (ModemInterface*)phonebook->priv;
        GValue *val = g_hash_table_lookup(modem->cache,
                                          GSMD_PHONEBOOK_KEY_DEFAULT_STORAGE_SPACE);

        if (val && !modem->no_cache) {
                modem->phonebook_ipc->get_default_storage_space_reply(modem->phonebook_ipc,
                                                                      ipc_data,
                                                                      g_value_get_string(val));
        } else {
                gsmd_modem_post_at_command_id( modem,
                                               PHONEBOOK_CUR_SPACE_QUERY,
                                               NULL,
                                               ipc_data,
                                               (ErrorFunction)modem->phonebook_ipc->get_default_storage_space_error,
                                               (gpointer)modem->phonebook_ipc,
                                               INTERFACE_PHONEBOOK,
                                               NULL);
        }
}


AtCommandHandlerStatus gsmd_phonebook_handler_set_default_storage_space(ModemInterface *modem,
                                                                        AtCommandContext *at,
                                                                        GString *response)
{
        AtCommandHandlerStatus status = AT_HANDLER_DONT_UNDERSTAND;
        GError **error = NULL;
        GRegex *regex = g_regex_new ("(?<error>ERROR)|(?<ok>OK)", 0, 0, error);
        GMatchInfo *match_info;


        if (g_regex_match (regex, response->str, 0, &match_info)) {
                if (gsmd_utils_match_exists(match_info,"error")) {
                        gsmd_utils_send_error(at,
                                              GSMD_ERROR_UNKNOWN,
                                              "FAILED TO SET DEFAULT PHONEBOOK STORAGE SPACE");
                        status = AT_HANDLER_DONE_ERROR;
                }

                if (gsmd_utils_match_exists(match_info,"ok")) {
                        //Since storage space changed, it's index boundaries
                        //might have changed aswell
                        gsmd_utils_table_insert_int(modem->cache,
                                                    GSMD_PHONEBOOK_KEY_LOWEST_INDEX,
                                                    9999);

                        gsmd_utils_table_insert_int(modem->cache,
                                                    GSMD_PHONEBOOK_KEY_HIGHEST_INDEX,
                                                    -9999);

                        GValue *val = g_hash_table_lookup(at->handler_data,GSMD_PHONEBOOK_KEY_DEFAULT_STORAGE_SPACE);

                        if (!val) {
                                g_warning("%s : Invalid handler data. default storage space: %p",
                                          __func__,
                                          val);

                                gsmd_utils_send_error(at,
                                                      GSMD_ERROR_UNKNOWN,
                                                      "INVALID HANDLER DATA");
                        }

                        gsmd_utils_table_insert_copy(modem->cache,
                                                     GSMD_PHONEBOOK_KEY_DEFAULT_STORAGE_SPACE,
                                                     val);

                        modem->phonebook_ipc->set_default_storage_space_reply(modem->phonebook_ipc,
                                                                              at->ipc_data);

                        status = AT_HANDLER_DONE;
                }

        }

        g_free(error);
        g_match_info_free (match_info);
        g_regex_unref (regex);


        return status;
}


void gsmd_phonebook_command_set_default_storage_space(PhonebookInterface *phonebook,
                                                      gpointer ipc_data,
                                                      const char *space)
{
        ModemInterface *modem = (ModemInterface*)phonebook->priv;

        GHashTable *data = gsmd_utils_create_hash_table();
        gsmd_utils_table_insert_string(data,
                                       GSMD_PHONEBOOK_KEY_DEFAULT_STORAGE_SPACE,
                                       space);

        gsmd_modem_post_at_command_id( modem,
                                       PHONEBOOK_SPACE_SET,
                                       space,
                                       ipc_data,
                                       (ErrorFunction)modem->phonebook_ipc->set_default_storage_space_error,
                                       (gpointer)modem->phonebook_ipc,
                                       INTERFACE_PHONEBOOK,
                                       data);
}

AtCommandHandlerStatus gsmd_phonebook_handler_get_index_boundaries(
        ModemInterface *modem,
        AtCommandContext *at,
        GString *response)
{
        /*
        AT+CPBR=?

        +CPBR: (1-250),20,20

        OK

        Regular expression: \+CPBR:\s\((\d+)-(\d+)\)
        */
        AtCommandHandlerStatus status = AT_HANDLER_DONT_UNDERSTAND;
        GError **error = NULL;
        GRegex *regex = g_regex_new ("\\+CPBR:\\s\\((?<min>\\d+)-(?<max>\\d+)\\)|(?<error>ERROR)|(?<ok>OK)", 0, 0, error);
        GMatchInfo *match_info;
        gchar *match = NULL;



        if (g_regex_match (regex, response->str, 0, &match_info)) {
                if (gsmd_utils_match_exists(match_info,"min")) {
                        match = g_match_info_fetch_named(match_info,"min");
                        gsmd_utils_table_insert_int(at->handler_data,GSMD_PHONEBOOK_KEY_LOWEST_INDEX,atoi(match));
                        g_free(match);

                        match = g_match_info_fetch_named(match_info,"max");
                        gsmd_utils_table_insert_int(at->handler_data,GSMD_PHONEBOOK_KEY_HIGHEST_INDEX,atoi(match));
                        g_free(match);



                        status = AT_HANDLER_NEED_MORE;
                }

                if (gsmd_utils_match_exists(match_info,"error")) {
                        //TODO should empty string be sent as a reply instead?
                        gsmd_utils_send_error(at,
                                              GSMD_ERROR_UNKNOWN,
                                              "UNABLE TO READ PHONEBOOK INDEX BOUNDARIES");
                        status = AT_HANDLER_DONE_ERROR;
                }

                if (gsmd_utils_match_exists(match_info,"ok")) {
                        GValue *min = g_hash_table_lookup(at->handler_data,GSMD_PHONEBOOK_KEY_LOWEST_INDEX);
                        GValue *max = g_hash_table_lookup(at->handler_data,GSMD_PHONEBOOK_KEY_HIGHEST_INDEX);

                        if (!min || !max) {
                                g_warning("%s : Invalid handler data. lowest_index: %p, highest_index: %p",
                                          __func__,
                                          min,
                                          max);

                                gsmd_utils_send_error(at,
                                                      GSMD_ERROR_UNKNOWN,
                                                      "INVALID HANDLER DATA");
                        }

                        modem->phonebook_ipc->get_index_boundaries_reply(modem->phonebook_ipc,
                                                                         at->ipc_data,
                                                                         g_value_get_int(min),
                                                                         g_value_get_int(max));


                        gsmd_utils_table_insert_copy(modem->cache,GSMD_PHONEBOOK_KEY_LOWEST_INDEX,min);
                        gsmd_utils_table_insert_copy(modem->cache,GSMD_PHONEBOOK_KEY_HIGHEST_INDEX,max);

                        status = AT_HANDLER_DONE;
                }

        }

        g_free(error);
        g_match_info_free (match_info);
        g_regex_unref (regex);


        return status;
}


void gsmd_phonebook_command_get_index_boundaries(PhonebookInterface *phonebook,
                                                 gpointer ipc_data)
{
        ModemInterface *modem = (ModemInterface*)phonebook->priv;
        GValue *min = g_hash_table_lookup(modem->cache,GSMD_PHONEBOOK_KEY_LOWEST_INDEX);
        GValue *max = g_hash_table_lookup(modem->cache,GSMD_PHONEBOOK_KEY_HIGHEST_INDEX);


        if (!modem->no_cache && min && max && g_value_get_int(min) < g_value_get_int(max)) {
                modem->phonebook_ipc->get_index_boundaries_reply(modem->phonebook_ipc,
                                                                 ipc_data,
                                                                 g_value_get_int(min),
                                                                 g_value_get_int(max));
        } else {
                gsmd_modem_post_at_command_id( modem,
                                               PHONEBOOK_GET_INDEX_BOUNDARIES,
                                               NULL,
                                               ipc_data,
                                               (ErrorFunction)modem->phonebook_ipc->delete_entry_error,
                                               (gpointer)modem->phonebook_ipc,
                                               INTERFACE_PHONEBOOK,
                                               NULL);
        }
}


AtCommandHandlerStatus gsmd_phonebook_handler_delete_entry(ModemInterface *modem,
                                                           AtCommandContext *at,
                                                           GString *response)
{
        AtCommandHandlerStatus status = AT_HANDLER_DONT_UNDERSTAND;
        GError **error = NULL;
        GRegex *regex = g_regex_new ("(?<error>ERROR)|(?<ok>OK)", 0, 0, error);
        GMatchInfo *match_info;


        if (g_regex_match (regex, response->str, 0, &match_info)) {
                if (gsmd_utils_match_exists(match_info,"error")) {
                        gsmd_utils_send_error(at,
                                              GSMD_ERROR_UNKNOWN,
                                              "FAILED TO DELETE PHONEBOOK ENTRY");
                        status = AT_HANDLER_DONE_ERROR;
                }

                if (gsmd_utils_match_exists(match_info,"ok")) {
                        modem->phonebook_ipc->delete_entry_reply(modem->phonebook_ipc,
                                                                 at->ipc_data);
                        status = AT_HANDLER_DONE;
                }

        }

        g_free(error);
        g_match_info_free (match_info);
        g_regex_unref (regex);


        return status;
}

void gsmd_phonebook_command_delete_entry(PhonebookInterface *phonebook,
                                         gpointer ipc_data,
                                         const int index)
{
        GString* message = g_string_new ("");
        g_string_printf(message, "%d",index);

        ModemInterface *modem = (ModemInterface*)phonebook->priv;
        gsmd_modem_post_at_command_id( modem,
                                       PHONEBOOK_ENTRY_DELETE,
                                       message->str,
                                       ipc_data,
                                       (ErrorFunction)modem->phonebook_ipc->delete_entry_error,
                                       (gpointer)modem->phonebook_ipc,
                                       INTERFACE_PHONEBOOK,
                                       NULL);
        g_string_free(message,TRUE);
}



AtCommandHandlerStatus gsmd_phonebook_handler_store_entry(
        ModemInterface *modem,
        AtCommandContext *at,
        GString *response)
{
        /*
        AT+CPBW=2,+35844123456789,145,Name

        OK
        */

        AtCommandHandlerStatus status = AT_HANDLER_DONT_UNDERSTAND;
        GError **error = NULL;
        GRegex *regex = g_regex_new ("(?<error>ERROR)|(?<ok>OK)", 0, 0, error);
        GMatchInfo *match_info;


        if (g_regex_match (regex, response->str, 0, &match_info)) {
                if (gsmd_utils_match_exists(match_info,"error")) {
                        gsmd_utils_send_error(at,
                                              GSMD_ERROR_UNKNOWN,
                                              "FAILED TO STORE PHONEBOOK ENTRY");
                        status = AT_HANDLER_DONE_ERROR;
                }

                if (gsmd_utils_match_exists(match_info,"ok")) {
                        modem->phonebook_ipc->store_entry_reply(modem->phonebook_ipc,
                                                                at->ipc_data);
                        status = AT_HANDLER_DONE;
                }

        }

        g_free(error);
        g_match_info_free (match_info);
        g_regex_unref (regex);


        return status;

}



void gsmd_phonebook_command_store_entry(PhonebookInterface *phonebook,
                                        gpointer ipc_data,
                                        const int index,
                                        const int type,
                                        const char *name,
                                        const char *contents)
{
        GString* message = g_string_new ("");
        g_string_printf(message, "%d,%s,%d,%s",index,contents,type,name);

        ModemInterface *modem = (ModemInterface*)phonebook->priv;
        gsmd_modem_post_at_command_id( modem,
                                       PHONEBOOK_ENTRY_SET,
                                       message->str,
                                       ipc_data,
                                       (ErrorFunction)modem->phonebook_ipc->store_entry_error,
                                       (gpointer)modem->phonebook_ipc,
                                       INTERFACE_PHONEBOOK,
                                       NULL);
        g_string_free(message,TRUE);
}


AtCommandHandlerStatus gsmd_phonebook_handler_retrieve_entry(ModemInterface *modem,
                                                             AtCommandContext *at,
                                                             GString *response)
{
        /*
        AT+CPBR=1

        +CPBR: 1,"040123456789",129,"Name"

        OK

        Regular expression: \+CPBR:\s(\d+),"(\w+)",(\d+),"(\w+)"
        */
        AtCommandHandlerStatus status = AT_HANDLER_DONT_UNDERSTAND;
        GError **error = NULL;
        GRegex *regex = g_regex_new ("\\+CPBR:\\s(?<id>\\d)+,\"(?<number>\\+?\\w+)\",(?<type>\\d+),\"(?<text>\\w+)\"|(?<error>ERROR)|(?<ok>OK)", 0, 0, error);

        GMatchInfo *match_info;


        if (g_regex_match (regex, response->str, 0, &match_info)) {
                if (gsmd_utils_match_exists(match_info,"id")) {

                        gchar *type = g_match_info_fetch_named(match_info,"type");
                        gchar *name = g_match_info_fetch_named(match_info,"text");
                        gchar *contents = g_match_info_fetch_named(match_info,"number");

                        //TODO only send this once we have ok
                        modem->phonebook_ipc->retrieve_entry_reply(modem->phonebook_ipc,
                                                                   at->ipc_data,
                                                                   atoi(type),
                                                                   name,
                                                                   contents);
                        g_free(type);
                        g_free(name);
                        g_free(contents);

                        status = AT_HANDLER_NEED_MORE;
                }

                if (gsmd_utils_match_exists(match_info,"error")) {
                        //TODO should empty string be sent as a reply instead?
                        gsmd_utils_send_error(at,
                                              GSMD_ERROR_UNKNOWN,
                                              "UNABLE TO READ PHONEBOOK INDEX BOUNDARIES");
                        status = AT_HANDLER_DONE_ERROR;
                }

                if (gsmd_utils_match_exists(match_info,"ok")) {
                        status = AT_HANDLER_DONE;
                }

        }

        g_free(error);
        g_match_info_free (match_info);
        g_regex_unref (regex);


        return status;
}



void gsmd_phonebook_command_retrieve_entry(PhonebookInterface *phonebook,
                                           gpointer ipc_data,
                                           const int index)
{
        GString* message = g_string_new ("");
        g_string_printf(message, "%d",index);

        ModemInterface *modem = (ModemInterface*)phonebook->priv;
        gsmd_modem_post_at_command_id( modem,
                                       PHONEBOOK_ENTRY_GET,
                                       message->str,
                                       ipc_data,
                                       (ErrorFunction)modem->phonebook_ipc->retrieve_entry_error,
                                       (gpointer)modem->phonebook_ipc,
                                       INTERFACE_PHONEBOOK,
                                       NULL);
        g_string_free(message,TRUE);
}



void gsmd_phonebook_init(ModemInterface *modem)
{
        gsmd_phonebook_init_at_handler(modem);

        modem->phonebook->priv = (gpointer)modem;
        modem->phonebook->retrieve_entry = &gsmd_phonebook_command_retrieve_entry;
        modem->phonebook->store_entry = &gsmd_phonebook_command_store_entry;
        modem->phonebook->delete_entry = &gsmd_phonebook_command_delete_entry;
        modem->phonebook->get_index_boundaries = &gsmd_phonebook_command_get_index_boundaries;
        modem->phonebook->set_default_storage_space = &gsmd_phonebook_command_set_default_storage_space;
        modem->phonebook->get_default_storage_space = &gsmd_phonebook_command_get_default_storage_space;
        modem->phonebook->get_storage_info = &gsmd_phonebook_command_get_storage_info;
        modem->phonebook->get_storage_spaces = &gsmd_phonebook_command_get_storage_spaces;

}

void gsmd_phonebook_deinitialize(ModemInterface *modem)
{
        gsmd_phonebook_free_storage_spaces_array(modem->cache);
}

