/*
 *  phonebook_interface.h
 *
 *  Copyright(C) 2007,2008 Ixonos Plc
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Boston, MA 02111.
 */

/*
 * Written by
 *   Jukka Honkela
 *   Yi Zheng
 *   Matti Katila
 *   Vesa Pikki
 *   Heikki Paajanen
 */


#ifndef PHONEBOOK_INTERFACE
#define PHONEBOOK_INTERFACE

typedef struct _PhonebookInterface PhonebookInterface;

/**
* @brief Phonebook interface
*/
struct _PhonebookInterface {
        gpointer priv;

        /**
         * @brief Gets a list of phonebooks storage spaces
         * @param modem modem pointer
         * @param ipc_data data for ipc
         */
        void (*get_storage_spaces) (PhonebookInterface *phonebook,
                                    gpointer ipc_data);


        /**
         * @brief Gets information on phonebooks storage
         * @param modem modem pointer
         * @param ipc_data data for ipc
         */
        void (*get_storage_info) (PhonebookInterface *phonebook,
                                  gpointer ipc_data);


        /**
         * @brief Gets default storage space of the phonebook
         * @param modem modem pointer
         * @param ipc_data data for ipc
         */
        void (*get_default_storage_space) (PhonebookInterface *phonebook,
                                           gpointer ipc_data);


        /**
         * @brief Sets default storage space for the phonebook
         * @param modem modem pointer
         * @param ipc_data data for ipc
         */
        void (*set_default_storage_space) (PhonebookInterface *phonebook,
                                           gpointer ipc_data,
                                           const char *space);


        /**
         * @brief Get phonebook's index boundaries
         * @param modem modem pointer
         * @param ipc_data data for ipc
         */
        void (*get_index_boundaries) (PhonebookInterface *phonebook,
                                      gpointer ipc_data);


        /**
         * @brief Deletes an entry from the phonebook
         * @param modem modem pointer
         * @param ipc_data data for ipc
         */
        void (*delete_entry) (PhonebookInterface *phonebook,
                              gpointer ipc_data,
                              const int index);


        /**
         * @brief Stores an entry to the phonebook
         * @param modem modem pointer
         * @param ipc_data data for ipc
         */
        void (*store_entry) (PhonebookInterface *phonebook,
                             gpointer ipc_data,
                             const int index,
                             const int type,
                             const char *name,
                             const char *contents);


        /**
         * @brief Retrieves an entry from the phonebook
         * @param modem modem pointer
         * @param ipc_data data for ipc
         */
        void (*retrieve_entry) (PhonebookInterface *phonebook,
                                gpointer ipc_data,
                                const int index);
};


typedef struct _PhonebookIPCInterface PhonebookIPCInterface;

struct _PhonebookIPCInterface {
        gpointer priv;

        /**************************Replies to method calls*************************/

        /**
         * @brief reply to "get_storage_spaces" method call
         * @param modem modem pointer
         * @param ipc_data data for ipc
         */
        void (*get_storage_spaces_reply) (PhonebookIPCInterface *phonebook_ipc,
                                          gpointer ipc_data,
                                          GArray *spaces);


        /**
         * @brief reply to "get_storage_info" method call
         * @param modem modem pointer
         * @param ipc_data data for ipc
         */
        void (*get_storage_info_reply) (PhonebookIPCInterface *phonebook_ipc,
                                        gpointer ipc_data,
                                        GArray *spaces);


        /**
         * @brief reply to "get_storage_spaces" method call
         * @param modem modem pointer
         * @param ipc_data data for ipc
         */
        void (*get_default_storage_space_reply) (PhonebookIPCInterface *phonebook_ipc,
                        gpointer ipc_data,
                        const char *space);


        /**
         * @brief reply to "set_default_storage_space" method call
         * @param modem modem pointer
         * @param ipc_data data for ipc
         */
        void (*set_default_storage_space_reply) (PhonebookIPCInterface *phonebook_ipc,
                        gpointer ipc_data);


        /**
         * @brief reply to "get_index_boundaries" method call
         * @param modem modem pointer
         * @param ipc_data data for ipc
         */
        void (*get_index_boundaries_reply) (PhonebookIPCInterface *phonebook_ipc,
                                            gpointer ipc_data,
                                            const int lowest_index,
                                            const int highest_index);


        /**
         * @brief reply to "delete_entry" method call
         * @param modem modem pointer
         * @param ipc_data data for ipc
         */
        void (*delete_entry_reply) (PhonebookIPCInterface *phonebook_ipc,
                                    gpointer ipc_data);


        /**
         * @brief reply to "store_entry" method call
         * @param modem modem pointer
         * @param ipc_data data for ipc
         */
        void (*store_entry_reply) (PhonebookIPCInterface *phonebook_ipc,
                                   gpointer ipc_data);


        /**
         * @brief reply to "retrieve_entry" method call
         * @param modem modem pointer
         * @param ipc_data data for ipc
         */
        void (*retrieve_entry_reply) (PhonebookIPCInterface *phonebook_ipc,
                                      gpointer ipc_data,
                                      const int type,
                                      const char *name,
                                      const char *contents);


        /*********************Error replies to method calls************************/

        /**
         * @brief Error reply to "get_storage_spaces" method call
         * @param modem modem pointer
         * @param ipc_data data for ipc
         */
        void (*get_storage_spaces_error) (PhonebookIPCInterface *phonebook_ipc,
                                          gpointer ipc_data,
                                          GError *error);


        /**
         * @brief Error reply to "get_storage_info" method call
         * @param modem modem pointer
         * @param ipc_data data for ipc
         */
        void (*get_storage_info_error) (PhonebookIPCInterface *phonebook_ipc,
                                        gpointer ipc_data,
                                        GError *error);


        /**
         * @brief Error reply to "get_default_storage_space" method call
         * @param modem modem pointer
         * @param ipc_data data for ipc
         */
        void (*get_default_storage_space_error) (PhonebookIPCInterface *phonebook_ipc,
                        gpointer ipc_data,
                        GError *error);


        /**
         * @brief Error reply to "set_default_storage_space" method call
         * @param modem modem pointer
         * @param ipc_data data for ipc
         */
        void (*set_default_storage_space_error) (PhonebookIPCInterface *phonebook_ipc,
                        gpointer ipc_data,
                        GError *error);


        /**
         * @brief Error reply to "get_index_boundaries" method call
         * @param modem modem pointer
         * @param ipc_data data for ipc
         */
        void (*get_index_boundaries_error) (PhonebookIPCInterface *phonebook_ipc,
                                            gpointer ipc_data,
                                            GError *error);


        /**
         * @brief Error reply to "delete_entry" method call
         * @param modem modem pointer
         * @param ipc_data data for ipc
         */
        void (*delete_entry_error) (PhonebookIPCInterface *phonebook_ipc,
                                    gpointer ipc_data,
                                    GError *error);


        /**
         * @brief Error reply to "store_entry" method call
         * @param modem modem pointer
         * @param ipc_data data for ipc
         */
        void (*store_entry_error) (PhonebookIPCInterface *phonebook_ipc,
                                   gpointer ipc_data,
                                   GError *error);


        /**
         * @brief Error reply to "retrieve_entry" method call
         * @param modem modem pointer
         * @param ipc_data data for ipc
         */
        void (*retrieve_entry_error) (PhonebookIPCInterface *phonebook_ipc,
                                      gpointer ipc_data,
                                      GError *error);

        /********************************Signals***********************************/


        /**
         * @brief callStarted signal is emitted when call
         * establish
         */
        void (*incoming_call) (PhonebookIPCInterface *phonebook_ipc,
                               const gchar*type,
                               const gchar *number,
                               const gint id);


};

#endif

