/*
 *  sim_inteface.h
 *
 *  Copyright(C) 2007,2008 Ixonos Plc
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Boston, MA 02111.
 */

/*
 * Written by
 *   Jukka Honkela
 *   Yi Zheng
 *   Matti Katila
 *   Vesa Pikki
 *   Heikki Paajanen
 */


#ifndef SIM_INTERFACE
#define SIM_INTERFACE

typedef struct _SIMInterface SIMInterface;

/**
 * @brief SIM interface
 */
struct _SIMInterface {

        gpointer priv;

        /***************************Method calls*******************************/
        /**
        * @brief Gets subscriber numbers
        *
        * @param modem pointer to modem struct
        * @param ipc_data ipc data
        */
        void (*get_subscriber_numbers) (SIMInterface *sim,
                                        gpointer ipc_data);

        /**
         * @brief Gets authorization status
         * @param sim sim interface pointer
         * @param ipc_data data for ipc
         */
        void (*get_auth_status) (SIMInterface *sim,
                                 gpointer ipc_data);

        /**
         * @brief Sends pin code
         * @param sim sim interface pointer
         * @param ipc_data data for ipc
         * @param pin pin code
         */
        void (*send_auth_code) (SIMInterface *sim,
                                gpointer ipc_data,
                                const char* pin);

        /**
         * @brief Unlocks sim card with puk code and sets a new pin code
         * @param sim sim interface pointer
         * @param ipc_data data for ipc
         * @param puk puk code
         * @param new_pin new pin code
         */
        void (*unlock) (SIMInterface *sim,
                        gpointer ipc_data,
                        const char* puk,
                        const char* new_pin);

        /**
         * @brief Changes pin code
         * @param sim sim interface pointer
         * @param ipc_data data for ipc
         * @param old_pin old pin code
         * @param new_pin new pin code
         */
        void (*change_auth_code) (SIMInterface *sim,
                                  gpointer ipc_data,
                                  const char* old_pin,
                                  const char* new_pin);

        /**
         * @brief Gets imsi (International Mobile Subscriber Identity) code
         * @param sim sim interface pointer
         * @param ipc_data data for ipc
         */
        void (*get_imsi) (SIMInterface *sim,
                          gpointer ipc_data);
        /**
        * @brief Gets service center number
        *
        * @param modem pointer to modem struct
        * @param ipc_data ipc data
        */
        void (*get_service_center_number) (SIMInterface *sim,
                                           gpointer ipc_data);
        /**
        * @brief Sets service center number
        *
        * @param modem pointer to modem struct
        * @param ipc_data ipc data
        * @param number service center number
        */
        void (*set_service_center_number) (SIMInterface *sim,
                                           gpointer ipc_data,
                                           const char *number);

        /**
        * @brief Gets country code
        *
        * @param modem pointer to modem struct
        * @param ipc_data ipc data
        */
        void (*get_country_code) (SIMInterface *sim,
                                  gpointer ipc_data);


        /**
        * @brief Stores a message
        *
        * @param modem pointer to modem struct
        * @param ipc_data ipc data
        * @param recipient messages recipient
        * @param contents messages contents
        */
        void (*store_message) (SIMInterface *sim,
                               gpointer ipc_data,
                               const gchar *message,
                               const gchar *number);

        /**
        * @brief Retrieves message
        *
        * @param modem pointer to modem struct
        * @param ipc_data ipc data
        * @param number message's index
        */
        void (*retrieve_message) (SIMInterface *sim,
                                  gpointer ipc_data,
                                  gint number);

        /**
        * @brief Gets phonebook info
        *
        * @param sms pointer to sms interface
        * @param ipc_data ipc data
        */
        void (*get_phonebook_info) (SIMInterface *sim,
                                    gpointer ipc_data);
        /**
        * @brief Gets messagebook info
        *
        * @param sms pointer to sms interface
        * @param ipc_data ipc data
        */
        void (*get_messagebook_info) (SIMInterface *sim,
                                      gpointer ipc_data);

        /**
        * @brief Deletes a phonebook entry
        *
        * @param sms pointer to sms interface
        * @param ipc_data ipc data
        * @param index of the message to sent
        */
        void (*delete_entry) (SIMInterface *sim,
                              gpointer ipc_data,
                              gint index);

        /**
        * @brief Stores a phonebook entry
        *
        * @param sms pointer to sms interface
        * @param ipc_data ipc data
        * @param index of the message to sent
        */
        void (*store_entry) (SIMInterface *sim,
                             gpointer ipc_data,
                             const gchar *recipient_number,
                             const gchar *contents,
                             gint index);

        /**
        * @brief Retrieves a phonebook entry
        *
        * @param sms pointer to sms interface
        * @param ipc_data ipc data
        * @param index of the message to sent
        */
        void (*retrieve_entry) (SIMInterface *sim,
                                gpointer ipc_data,
                                gint index);

        /**
        * @brief Deletes a message
        *
        * @param sms pointer to sms interface
        * @param ipc_data ipc data
        * @param index of the message to delete
        */
        void (*delete_message) (SIMInterface *sim,
                                gpointer ipc_data,
                                gint index);

        /**
        * @brief Sends a message from sim card
        *
        * @param sms pointer to sms interface
        * @param ipc_data ipc data
        * @param index of the message to delete
        */
        void (*send_message) (SIMInterface *sim,
                              gpointer ipc_data,
                              gint index,
                              gboolean want_report);



        /**
        * @brief Lists messages from sim card
        *
        * @param sms pointer to sms interface
        * @param ipc_data ipc data
        * @param status status of messages to list
        * ("new","read","outbox","sent","all")
        */
        void (*list_messages) (SIMInterface *sim,
                               gpointer ipc_data,
                               const gchar *status);
};

typedef struct _SIMIPCInterface SIMIPCInterface;


struct _SIMIPCInterface {

        gpointer priv;

        /**************************Replies to method calls*************************/
        /**
        * @brief Reply to "get_subscriber_numbers" method call
        *
        * @param modem pointer to modem struct
        * @param ipc_data ipc data
        * @param numbers subscriber numbers
        */
        void (*get_subscriber_numbers_reply) (SIMIPCInterface *sim_ipc,
                                              gpointer ipc_data,
                                              const char **numbers);

        /**
          * @brief Reply to "get_auth_status" method call
          * @param sim_ipc sim IPC interface pointer
          * @param ipc_data data for ipc
          * @param status authorization status
          */
        void (*get_auth_status_reply) (SIMIPCInterface *sim_ipc,
                                       gpointer ipc_data,
                                       const char *status);


        /**
         * @brief Reply to "send_auth_code" method call
         * @param sim_ipc sim IPC interface pointer
         * @param ipc_data data for ipc
         */
        void (*send_auth_code_reply) (SIMIPCInterface *sim_ipc,
                                      gpointer ipc_data);

        /**
         * @brief Reply to "unlock" method call
         * @param sim_ipc sim IPC interface pointer
         * @param ipc_data data for ipc
         */
        void (*unlock_reply) (SIMIPCInterface *sim_ipc,
                              gpointer ipc_data);

        /**
         * @brief Reply to "change_auth_code" method call
         * @param sim_ipc sim IPC interface pointer
         * @param ipc_data data for ipc
         */
        void (*change_auth_code_reply) (SIMIPCInterface *sim_ipc,
                                        gpointer ipc_data);

        /**
         * @brief Reply to "get_imsi" method call
         * @param sim_ipc sim IPC interface pointer
         * @param ipc_data data for ipc
         * @param imsi imsi code
         */
        void (*get_imsi_reply) (SIMIPCInterface *sim_ipc,
                                gpointer ipc_data,
                                const char *imsi);
        /**
        * @brief Reply to "get service center number" method call
        *
        * @param modem pointer to modem struct
        * @param ipc_data ipc data
        * @param number service center number
        */
        void (*get_service_center_number_reply) (SIMIPCInterface *sim_ipc,
                                                 gpointer ipc_data,
                                                 const char *number);

        /**
        * @brief Reply to "set service center number" method call
        *
        * @param modem pointer to modem struct
        * @param ipc_data ipc data
        */
        void (*set_service_center_number_reply) (SIMIPCInterface *sim_ipc,
                                                 gpointer ipc_data);

        /**
        * @brief Reply to "get_country_code" method call
        *
        * @param modem pointer to modem struct
        * @param ipc_data ipc data
        * @param dial_code
        */
        void (*get_country_code_reply) (SIMIPCInterface *sim_ipc,
                                        gpointer ipc_data,
                                        const char *dial_code);


        /**
        * @brief Reply to "store a message" method call
        *
        * @param modem pointer to modem struct
        * @param ipc_data ipc data
        * @param index stored message's index
        */
        void (*store_message_reply) (SIMIPCInterface *sim_ipc,
                                     gpointer ipc_data,
                                     const int index);

        /**
        * @brief Reply to "retrieve message" method call
        *
        * @param modem pointer to modem struct
        * @param ipc_data ipc data
        * @param sender_number message sender's number
        * @param content messages contents
        */
        void (*retrieve_message_reply) (SIMIPCInterface *sim_ipc,
                                        gpointer ipc_data,
                                        const char *sender_number,
                                        const char *content,
                                        gint time);

        /**
        * @brief Reply to "get phonebook info" method call
        *
        * @param modem pointer to modem struct
        * @param ipc_data ipc data
        */
        void (*get_phonebook_info_reply) (SIMIPCInterface *sim_ipc,
                                          gpointer ipc_data,
                                          GHashTable *info);

        /**
        * @brief Reply to "get messagebook info" method call
        *
        * @param modem pointer to modem struct
        * @param ipc_data ipc data
        */
        void (*get_messagebook_info_reply) (SIMIPCInterface *sim_ipc,
                                            gpointer ipc_data,
                                            GHashTable *info);

        /**
        * @brief Reply to "delete entry" method call
        *
        * @param modem pointer to modem struct
        * @param ipc_data ipc data
        */
        void (*delete_entry_reply) (SIMIPCInterface *sim_ipc,
                                    gpointer ipc_data);

        /**
        * @brief Reply to "store entry" method call
        *
        * @param modem pointer to modem struct
        * @param ipc_data ipc data
        */
        void (*store_entry_reply) (SIMIPCInterface *sim_ipc,
                                   gpointer ipc_data);
        /**
        * @brief Reply to "retrieve entry" method call
        *
        * @param modem pointer to modem struct
        * @param ipc_data ipc data
        */
        void (*retrieve_entry_reply) (SIMIPCInterface *sim_ipc,
                                      gpointer ipc_data,
                                      const gchar *number,
                                      const gchar *content);

        /**
        * @brief Reply to "delete entry" method call
        *
        * @param modem pointer to modem struct
        * @param ipc_data ipc data
        */
        void (*delete_message_reply) (SIMIPCInterface *sim_ipc,
                                      gpointer ipc_data);

        /**
        * @brief Reply to "list messages" method call
        *
        * @param modem pointer to modem struct
        * @param ipc_data ipc data
        */
        void (*list_messages_reply) (SIMIPCInterface *sim_ipc,
                                     gpointer ipc_data,
                                     GArray *indices);

        /**
        * @brief Reply to "send messages" method call
        *
        * @param modem pointer to modem struct
        * @param ipc_data ipc data
        */
        void (*send_message_reply) (SIMIPCInterface *sim_ipc,
                                    gpointer ipc_data);

        /*********************Error replies to method calls********************/
        /**
        * @brief Error reply to "get_subscriber_numbers" method call
        *
        * @param modem pointer to modem struct
        * @param ipc_data ipc data
        * @param index index of the provider to register to
        */
        void (*get_subscriber_numbers_error) (SIMIPCInterface *sim_ipc,
                                              gpointer ipc_data,
                                              GError *errors);

        /**
        * @brief Error reply to "get service center number" method call
        *
        * @param modem pointer to modem struct
        * @param ipc_data ipc data
        * @param error pointer to GError describing error
        */
        void (*get_service_center_number_error) (SIMIPCInterface *sim_ipc,
                                                 gpointer ipc_data,
                                                 GError *error);
        /**
        * @brief Error reply to "set service center number" method call
        *
        * @param modem pointer to modem struct
        * @param ipc_data ipc data
        * @param error pointer to GError describing error
        */
        void (*set_service_center_number_error) (SIMIPCInterface *sim_ipc,
                                                 gpointer ipc_data,
                                                 GError *error);

        /**
         * @brief Error reply to "get_auth_status" method call
         * @param sim_ipc sim IPC interface pointer
         * @param ipc_data data for ipc
         */
        void (*get_auth_status_error) (SIMIPCInterface *sim_ipc,
                                       gpointer ipc_data,
                                       GError *error);



        /**
         * @brief Error reply to "send_auth_code" method call
         * @param sim_ipc sim IPC interface pointer
         * @param ipc_data data for ipc
         */
        void (*send_auth_code_error) (SIMIPCInterface *sim_ipc,
                                      gpointer ipc_data,
                                      GError *error);

        /**
         * @brief Error reply to "unlock" method call
         * @param sim_ipc sim IPC interface pointer
         * @param ipc_data data for ipc
         */
        void (*unlock_error) (SIMIPCInterface *sim_ipc,
                              gpointer ipc_data,
                              GError *error);

        /**
         * @brief Error reply to "change_auth_code" method call
         * @param sim_ipc sim IPC interface pointer
         * @param ipc_data data for ipc
         */
        void (*change_auth_code_error) (SIMIPCInterface *sim_ipc,
                                        gpointer ipc_data,
                                        GError *error);

        /**
         * @brief Error reply to "get_imsi" method call
         * @param sim_ipc sim IPC interface pointer
         * @param ipc_data data for ipc
         * @param imsi imsi code
         */
        void (*get_imsi_error) (SIMIPCInterface *sim_ipc,
                                gpointer ipc_data,
                                GError *error);

        /**
        * @brief Error reply to "get_country_code" method call
        *
        * @param modem pointer to modem struct
        * @param ipc_data ipc data
        */
        void (*get_country_code_error) (SIMIPCInterface *sim_ipc,
                                        gpointer ipc_data,
                                        GError *error);

        /**
        * @brief Error reply to "store a message" method call
        *
        * @param modem pointer to modem struct
        * @param ipc_data ipc data
        * @param error pointer to GError describing error
        */
        void (*store_message_error) (SIMIPCInterface *sim_ipc,
                                     gpointer ipc_data,
                                     GError *error);

        /**
        * @brief Error reply to "retrieve message" method call
        *
        * @param modem pointer to modem struct
        * @param ipc_data ipc data
        * @param error pointer to GError describing error
        */
        void (*retrieve_message_error) (SIMIPCInterface *sim_ipc,
                                        gpointer ipc_data,
                                        GError *error);

        /**
        * @brief Error reply to "get phonebook info" method call
        *
        * @param modem pointer to modem struct
        * @param ipc_data ipc data
        * @param error pointer to GError describing error
        */
        void (*get_phonebook_info_error) (SIMIPCInterface *sim_ipc,
                                          gpointer ipc_data,
                                          GError *error);

        /**
        * @brief Error reply to "get messagebook info" method call
        *
        * @param modem pointer to modem struct
        * @param ipc_data ipc data
        * @param error pointer to GError describing error
        */
        void (*get_messagebook_info_error) (SIMIPCInterface *sim_ipc,
                                            gpointer ipc_data,
                                            GError *error);

        /**
        * @brief Error reply to "delete entry" method call
        *
        * @param modem pointer to modem struct
        * @param ipc_data ipc data
        * @param error pointer to GError describing error
        */
        void (*delete_entry_error) (SIMIPCInterface *sim_ipc,
                                    gpointer ipc_data,
                                    GError *error);

        /**
        * @brief Error reply to "store entry" method call
        *
        * @param modem pointer to modem struct
        * @param ipc_data ipc data
        * @param error pointer to GError describing error
        */
        void (*store_entry_error) (SIMIPCInterface *sim_ipc,
                                   gpointer ipc_data,
                                   GError *error);

        /**
        * @brief Error reply to "retreve entry" method call
        *
        * @param modem pointer to modem struct
        * @param ipc_data ipc data
        * @param error pointer to GError describing error
        */
        void (*retrieve_entry_error) (SIMIPCInterface *sim_ipc,
                                      gpointer ipc_data,
                                      GError *error);

        /**
        * @brief Error reply to "delete entry" method call
        *
        * @param modem pointer to modem struct
        * @param ipc_data ipc data
        * @param error pointer to GError describing error
        */
        void (*delete_message_error) (SIMIPCInterface *sim_ipc,
                                      gpointer ipc_data,
                                      GError *error);

        /**
        * @brief Error reply to "send message" method call
        *
        * @param modem pointer to modem struct
        * @param ipc_data ipc data
        * @param error pointer to GError describing error
        */
        void (*send_message_error) (SIMIPCInterface *sim_ipc,
                                    gpointer ipc_data,
                                    GError *error);

        /**
        * @brief Error reply to "list messages" method call
        *
        * @param modem pointer to modem struct
        * @param ipc_data ipc data
        * @param error pointer to GError describing error
        */
        void (*list_messages_error) (SIMIPCInterface *sim_ipc,
                                     gpointer ipc_data,
                                     GError *error);
        /********************************Signals***********************************/


        /**
         * @brief Authorization status signal
         * @param sim_ipc sim IPC interface pointer
         * @param status authorization stauts
         */
        void (*auth_status) (SIMIPCInterface *sim_ipc,
                             const char *status);

};

#endif

